#include <konkret/konkret.h>
#include "Linux_Group.h"

#include <libuser/entity.h>
#include <libuser/user.h>

#include "aux_lu.h"
#include "globals.h"

static const CMPIBroker* _cb = NULL;

static void Linux_GroupInitialize()
{
}

static CMPIStatus Linux_GroupCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_GroupEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Linux_GroupEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Linux_Group lg;
    struct lu_context *luc = NULL;
    struct lu_error *error = NULL;
    GPtrArray *groups = NULL;
    struct lu_ent *lue = NULL;
    size_t i;
    const char *nameSpace = KNameSpace(cop);
    const char *hostname = get_system_name();
    char *instanceid = NULL;

    luc = lu_start(NULL, lu_group, NULL, NULL, lu_prompt_console_quiet, NULL, &error);
    if (!luc)
      {
        fprintf(stderr, "Error initializing: %s\n", lu_strerror(error));
        exit(1);
      }

    instanceid = malloc(256 * sizeof(char));
    groups = lu_groups_enumerate_full(luc, "*", &error);
    for (i = 0;  (groups != NULL) && (i < groups->len); i++)
      {
        lue = g_ptr_array_index(groups, i);

        Linux_Group_Init(&lg, _cb, nameSpace);
        Linux_Group_Set_CreationClassName(&lg, LGCCLASSNAME);
        Linux_Group_Set_Name(&lg, aux_lu_get_str(lue, LU_GROUPNAME));
        Linux_Group_Set_ElementName(&lg, aux_lu_get_str(lue, LU_GROUPNAME));
        Linux_Group_Set_CommonName(&lg, aux_lu_get_str(lue, LU_GROUPNAME));
        snprintf(instanceid, 255, ORGID":GID:%ld",
          aux_lu_get_long(lue, LU_GIDNUMBER));
        Linux_Group_Set_InstanceID(&lg, instanceid);
        KReturnInstance(cr, lg);
        lu_ent_free(lue);
      } /* for */
    free(instanceid);

    if (groups)
      {
        g_ptr_array_free(groups, TRUE);
      }

    lu_end(luc);

    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_GroupGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Linux_GroupCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_GroupModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_GroupDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_GroupExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    Linux_Group,
    Linux_Group,
    _cb,
    Linux_GroupInitialize())

static CMPIStatus Linux_GroupMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_GroupInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return Linux_Group_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    Linux_Group,
    Linux_Group,
    _cb,
    Linux_GroupInitialize())

KONKRET_REGISTRATION(
    "root/cimv2",
    "Linux_Group",
    "Linux_Group",
    "instance method");
