#include <konkret/konkret.h>
#include "Linux_HostedService.h"
#include "Linux_ComputerSystem.h"
#include "Linux_AccountManagementService.h"

#include "globals.h"

static const CMPIBroker* _cb;

static void Linux_HostedServiceInitialize()
{
}

static CMPIStatus Linux_HostedServiceCleanup( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc, 
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_HostedServiceEnumInstanceNames( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Linux_HostedServiceEnumInstances( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char** properties) 
{
    Linux_ComputerSystemRef lcsref;
    Linux_AccountManagementServiceRef lamsref;
    Linux_HostedService lhs;

    const char *nameSpace = KNameSpace(cop);
    const char *hostname = get_system_name();

    Linux_ComputerSystemRef_Init(&lcsref, _cb, nameSpace);
    Linux_ComputerSystemRef_Set_Name(&lcsref, hostname);
    Linux_ComputerSystemRef_Set_CreationClassName(&lcsref,
      Linux_ComputerSystem_ClassName);

    Linux_AccountManagementServiceRef_Init(&lamsref, _cb, nameSpace);
    Linux_AccountManagementServiceRef_Set_Name(&lamsref, LAMSNAME);
    Linux_AccountManagementServiceRef_Set_SystemCreationClassName(&lamsref,
      SCCLASSNAME);
    Linux_AccountManagementServiceRef_Set_SystemName(&lamsref, hostname);
    Linux_AccountManagementServiceRef_Set_CreationClassName(&lamsref,
      Linux_AccountManagementService_ClassName);

    Linux_HostedService_Init(&lhs, _cb, nameSpace);
    Linux_HostedService_Set_Antecedent(&lhs, &lcsref);
    Linux_HostedService_Set_Dependent(&lhs, &lamsref);

    KReturnInstance(cr, lhs);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_HostedServiceGetInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc,
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char** properties) 
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Linux_HostedServiceCreateInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const CMPIInstance* ci) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_HostedServiceModifyInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop,
    const CMPIInstance* ci, 
    const char**properties) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_HostedServiceDeleteInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_HostedServiceExecQuery(
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char* lang, 
    const char* query) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_HostedServiceAssociationCleanup( 
    CMPIAssociationMI* mi,
    const CMPIContext* cc, 
    CMPIBoolean term) 
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_HostedServiceAssociators(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole,
    const char** properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_HostedService_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus Linux_HostedServiceAssociatorNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_HostedService_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus Linux_HostedServiceReferences(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role,
    const char** properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_HostedService_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus Linux_HostedServiceReferenceNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_HostedService_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub( 
    Linux_HostedService,
    Linux_HostedService,
    _cb,
    Linux_HostedServiceInitialize())

CMAssociationMIStub( 
    Linux_HostedService,
    Linux_HostedService,
    _cb,
    Linux_HostedServiceInitialize())

KONKRET_REGISTRATION(
    "root/cimv2",
    "Linux_HostedService",
    "Linux_HostedService",
    "instance association");
